@@ -9,7 +9,7 @@ pub struct BuildConfig {
     pub(crate) file_name: Arc<PathBuf>,
     pub(crate) dir_of_code: Arc<PathBuf>,
     pub(crate) manifest_path: Arc<PathBuf>,
-    pub(crate) use_ir: bool,
+    pub(crate) use_orig_asm: bool,
     pub(crate) print_intermediate_asm: bool,
     pub(crate) print_finalized_asm: bool,
     pub(crate) print_ir: bool,
@@ -29,7 +29,7 @@ impl BuildConfig {
             file_name: Arc::new(file_name),
             dir_of_code: Arc::new(path),
             manifest_path: Arc::new(canonicalized_manifest_path),
-            use_ir: false,
+            use_orig_asm: false,
             print_intermediate_asm: false,
             print_finalized_asm: false,
             print_ir: false,
@@ -37,8 +37,11 @@ impl BuildConfig {
         }
     }
 
-    pub fn use_ir(self, a: bool) -> Self {
-        Self { use_ir: a, ..self }
+    pub fn use_orig_asm(self, a: bool) -> Self {
+        Self {
+            use_orig_asm: a,
+            ..self
+        }
     }
 
     pub fn print_intermediate_asm(self, a: bool) -> Self {
