@@ -1,10 +1,4 @@
-use crate::{
-    error::*,
-    parse_tree::{ident, Expression},
-    parser::Rule,
-    type_engine::*,
-    BuildConfig,
-};
+use crate::{error::*, parse_tree::ident, parser::Rule, type_engine::*, BuildConfig};
 
 use sway_types::{ident::Ident, span::Span};
 
@@ -26,7 +20,6 @@ pub struct StorageDeclaration {
 pub struct StorageField {
     pub name: Ident,
     pub r#type: TypeInfo,
-    pub initializer: Expression,
 }
 
 impl StorageField {
@@ -39,7 +32,6 @@ impl StorageField {
         let mut iter = pair.into_inner();
         let name = iter.next().expect("guaranteed by grammar");
         let r#type = iter.next().expect("guaranteed by grammar");
-        let initializer = iter.next().expect("guaranteed by grammar");
 
         let name = check!(
             ident::parse_from_pair(name, conf),
@@ -53,20 +45,10 @@ impl StorageField {
             warnings,
             errors
         );
-        let initializer_result = check!(
-            Expression::parse_from_pair(initializer, conf),
-            return err(warnings, errors),
-            warnings,
-            errors
-        );
-        let res = StorageField {
-            name,
-            r#type,
-            initializer: initializer_result.value.clone(),
-        };
+        let res = StorageField { name, r#type };
         ok(
             ParserLifter {
-                var_decls: initializer_result.var_decls,
+                var_decls: vec![],
                 value: res,
             },
             warnings,
